/**
 * Lawnchair StratifiedJS Module
 * =============================
 * A lightweight JSON document store.
 *
 * exports.init(opts)
 *   opts.path: path to the adaptors directory ("adaptors/")
 *   opts.adaptor: the shortname of the adaptor to be included and used ("dom")
 */
 
exports.init = function (opts) { return new Lawnchair(opts); };
 
var Lawnchair = function(opts) {
  this.init(opts);
}

Lawnchair.prototype = {
  
  init:function(opts) {
    opts = opts || {};
    opts.path = opts.path || "adaptors/";
    opts.adaptor = opts.adaptor || "dom";
    
    var adaptors = {
      'webkit': 'WebkitSQLiteAdaptor',
      'gears': 'GearsSQLiteAdaptor',
      'dom': 'DOMStorageAdaptor',
      'cookie': 'CookieAdaptor',
      'air': 'AIRSQLiteAdaptor',
      'userdata': 'UserDataAdaptor',
      'air-async': 'AIRSQLiteAsyncAdaptor',
      'blackberry': 'BlackBerryPersistentStorageAdaptor',
      'couch': 'CouchAdaptor'
    };

    var name = adaptors[opts.adaptor];
    var http = require("apollo:http"); // external apollo module hub
    http.script(opts.path + "LawnchairAdaptorHelpers.js");
    http.script(opts.path + name + ".js");
    this.adaptor = new window[name](opts);
    
    // Check for native JSON functions.
    if (!JSON || !JSON.stringify) throw "Native JSON functions unavailable - please include http://www.json.org/json2.js or run on a decent browser :P";
  },
  
  // Save an object to the store. If a key is present then update. Otherwise create a new record.
  save:function(obj) { waitfor(var rv) { this.adaptor.save(obj, resume); } return rv; },
  
  // Invokes a callback on an object with the matching key.
  get:function(key) { waitfor(var rv) { this.adaptor.get(key, resume); } return rv; },

  // Returns whether a key exists to a callback.
  exists:function() { waitfor(var rv) { this.adaptor.exists(resume); } return rv; },
  
  // Returns all rows to a callback.
  all:function() { waitfor(var rv) { this.adaptor.all(resume); } return rv; },
  
  // Removes a json object from the store.
  remove:function(keyOrObj) {  waitfor() { this.adaptor.remove(keyOrObj, resume); } },
  
  // Removes all documents from a store and returns self.
  nuke:function() {  waitfor() { this.adaptor.nuke(resume); } return this},
  
  // Returns a page of results based on offset provided by user and perPage option
  paged:function(page) {  waitfor(var rv) { this.adaptor.paged(page, resume); } return rv; },
  
  //Find method that accepts a conditional test for a record (method or eval string)
  find:function(condition) {
    var is = (typeof condition == 'string') ? function(r){return eval(condition)} : condition;
    var results = [];
    this.each(function(record, index) {
      if (is(record)) results.push(record);
    });
    return results;
  },
  
  /**
   * Iterator that accepts two paramters (methods or eval strings):
   *
   * - conditional test for a record
   * - callback to invoke on matches
   *
   */
  findIterator:function(condition, callback) {
    var cb = this.adaptor.terseToVerboseCallback(callback);
    var results = this.find(condition);
    var l = results.length;
    for (var i = 0; i < l; i++) {
      cb(results[i], i);
    }
  },


  /**
   * Classic iterator.
   * - Passes the record and the index as the second parameter to the callback.
   * - Accepts a string for eval or a method to be invoked for each document in the collection.
   */
  each:function(callback) {
    var cb = this.adaptor.terseToVerboseCallback(callback);
    var results = this.all();
    var l = results.length;
    for (var i = 0; i < l; i++) {
      cb(results[i], i);
    }
  }
// --
};
