// Start Class Definition...

GTDTask.prototype.SYNCSTATENEW = "NEW";
GTDTask.prototype.SYNCSTATENORMAL = "NORMAL";
GTDTask.prototype.SYNCSTATEDELETED = "DELETED";

GTDCategory.prototype.SYNCSTATENEW = "NEW";
GTDCategory.prototype.SYNCSTATENORMAL = "NORMAL";
GTDCategory.prototype.SYNCSTATEDELETED = "DELETED";


GTDCategoryCollection.UNCATEGORIZEDID = "_UNCATEGORIZED";

function GTDTask(id){

	this._title = null;
	this._note = null;
	this._id = null;
	this._done= false;
	this._category = null;
	this._syncState = "NEW";
	this._creationTime = spurious_abstractor.dateNowUTC();
	this._isNote = false;

   if (id != null) {
		this._id = id;
   } else {
		this._id = spurious_common.randomString();
   }
}



GTDTask.prototype.setIsNote = function(b) {
	
	this._isNote = b;
	
}

GTDTask.prototype.isNote = function() {
	
	return this._isNote;
	
}

GTDTask.prototype.makeNewID = function() {
	
	this._id = spurious_common.randomString();
	
}


GTDTask.prototype.setSyncStateToNormal = function() {
	
	this._syncState = GTDTask.prototype.SYNCSTATENORMAL;
	
}

GTDTask.prototype.setSyncStateToDeleted = function() {
	
	this._syncState = GTDTask.prototype.SYNCSTATEDELETED;
	
}

GTDTask.prototype.setSyncStateToNew = function() {

	this._syncState = GTDTask.prototype.SYNCSTATENEW;
	
}


GTDTask.prototype.isNew = function() {

	return (this.syncState() == GTDTask.prototype.SYNCSTATENEW);

}

GTDTask.prototype.isNormal = function() {

	return (this.syncState() == GTDTask.prototype.SYNCSTATENORMAL);

}

GTDTask.prototype.syncState = function() {
	
	if (this._syncState == null) {
			this._syncState = GTDTask.prototype.SYNCSTATENEW;
	}
	
	return this._syncState;
	
}


GTDTask.prototype.initFromLookup = function(lookup){

	this._title = lookup._title;
	this._note = lookup._note;
	this._id = lookup._id;
	
	
	if (lookup._done == "false" || lookup._done == "0") {
		this._done = false;
	} else {
		this._done = true;
	}
	
	
	if (lookup._isNote != null) {
		if (lookup._isNote == "true" || lookup._isNote == "1" || lookup._isNote == 1) {
			this._isNote = true;
		}
	} else {
		this._isNote = false;
	}
	
	
	this._categoryID = lookup._categoryID;
	this._syncState = lookup._syncState;
	
	if (lookup._creationTime != null) {
		this._creationTime = lookup._creationTime;
	} else {
		this._creationTime = spurious_abstractor.dateNowUTC();
	}

}

GTDTask.prototype.categoryID = function(){
	return this._categoryID;
}


GTDTask.prototype.asJSONForSync = function(){
	var json_data = JSON.stringify(this.asLookup());
	spurious_abstractor.trace(json_data);
	return json_data;
}


GTDTask.prototype.creationTimeInSeconds = function() {
	
	spurious_abstractor.trace("creationTimeInSeconds called");
	
	
	var d = new Date(this.creationTime());
	var res = d.getTime();
	spurious_abstractor.trace("CRT: "+res);
	
	return res;
}

GTDTask.prototype.creationTime = function() {
	return this._creationTime;
}

GTDTask.prototype.asLookup = function(){

	lookup = {};
	lookup["_syncState"] = this.syncState();
	lookup["_title"] = this._title;
	lookup["_note"]  = this._note;
	lookup["_id"]  = this._id;
	lookup["_done"] = this._done;
	lookup["_categoryID"] = this._categoryID;
	lookup["_creationTime"] = this.creationTime();
	lookup["_isNote"] = this.isNote();
	
	//for (key in lookup) {
	//	spurious_abstractor.trace("GTDTask Lookup: "+key);
	//}

	return lookup;
	
}

GTDTask.prototype.parentCategory = function() {

	return spurious_main.categoryCollection().categoryForID(this._categoryID);
	
}



GTDTask.prototype.markAsNotDoneNoCommit = function() {

	this._done = false;

}

GTDTask.prototype.markAsNotDone = function() {
	this._done = false;
	this.setSyncStateToNew();
	this.commit();
}


GTDTask.prototype.doneState = function() {
	return this._done;
}

GTDTask.prototype.toggleTaskDone = function() {

	this._done = !this._done;
	this.setSyncStateToNew();
	this.commit();

}

GTDTask.prototype.markAsDone = function() {
	this._done = true;
	this.setSyncStateToNew();
	this.commit();
}

GTDTask.prototype.actuallyErase = function() {
	spurious_lawnchair.deleteTask(this.id());
	this.parentCategory().removeTaskWithID(this.id());
}

GTDTask.prototype.erase = function() {
	this.setSyncStateToDeleted();
	this.parentCategory().removeTaskWithID(this.id());
	this.commit();
}
	
	

GTDTask.prototype.commit = function(){
	spurious_lawnchair.deleteTask(this.id());
	spurious_lawnchair.tasksTable().save(this.asLookup());
	spurious_main.setDirty(1);
}

GTDTask.prototype.setTitle = function(text) {

	this._title = text;

}

GTDTask.prototype.setCategoryID = function(catid) {

	this._categoryID = catid;

}

GTDTask.prototype.setNote = function(text) {

	this._note = text;

}


GTDTask.prototype.title = function() {

	return this._title;

}

GTDTask.prototype.note = function() {

	return this._note;

}

GTDTask.prototype.id = function() {

	return this._id;

}

//End Class Definition



// Start Class Definition...
function GTDCategory(id_in){

	this._prettyName = null;
	this._id = null;
	this._tasks = [];
	this._syncState;
	this._categoryCollection = null;
	this._sharedFromUser = null;


   if (id_in != null) {
		this._id = id_in;
   } else {
		this._id = spurious_common.randomString();
   }
}


GTDCategory.prototype.initFromLookup = function(lookup){

	this._prettyName = lookup._prettyName;
	this._id = lookup._id;
	this._syncState = lookup._syncState;

	this._sharedFromUser = lookup._sharedFromUser;
	
	spurious_abstractor.trace("SHARED FROM USER: "+this._sharedFromUser);

}


GTDCategory.prototype.isNew = function() {

	return (this.syncState() == GTDCategory.prototype.SYNCSTATENEW);

}



GTDCategory.prototype.isNormal = function() {

	return (this.syncState() == GTDCategory.prototype.SYNCSTATENORMAL);

}

GTDCategory.prototype.syncState = function() {
	
	if (this._syncState == null) {
			this._syncState = "NEW";
	}
	
	return this._syncState;
	
}



GTDCategory.prototype.asJSONForSyncBase64 = function(){
	var json_data = JSON.stringify(this.asLookup());
	return spurious_sync.base64_encode(json_data);
}


GTDCategory.prototype.asJSONForSync = function(){
	var json_data = JSON.stringify(this.asLookup());
	return json_data;
}


GTDCategory.prototype.asLookup = function(){
	
	lookup = {};
	
	lookup["_prettyName"] = this.prettyName();
	lookup["_id"] = this._id;
	lookup["_syncState"] = this.syncState();
	lookup["_sharedFromUser"] = this.sharedFromUser();
	
	
	return lookup;
	
}

GTDCategory.prototype.commit = function(){	
	spurious_lawnchair.deleteCategory(this.id(),false);
	spurious_lawnchair.categoriesTable().save(this.asLookup());
	spurious_main.setDirty(1);
}


GTDCategory.prototype.setCategoryCollection = function(cc) {
	this._categoryCollection = cc;
}

GTDCategory.prototype.categoryCollection = function() {
	return this._categoryCollection;
}



GTDCategory.prototype.setSyncStateToNew = function() {
	
	this._syncState = GTDCategory.prototype.SYNCSTATENEW;
	
}

GTDCategory.prototype.setSyncStateToNormal = function() {
	
	this._syncState = GTDCategory.prototype.SYNCSTATENORMAL;
	
}

GTDCategory.prototype.setSyncStateToDeleted = function() {
	
	this._syncState = GTDCategory.prototype.SYNCSTATEDELETED;
	
}


GTDCategory.prototype.actuallyErase = function() {


	if (this.id() == "_UNCATEGORIZED") {
		return;
	}

	spurious_lawnchair.deleteCategory(this.id());
	this.categoryCollection().removeCategoryWithID(this.id());
}

GTDCategory.prototype.erase = function() {

	var alltasks = this.allTaskIDs();
	
	for (var taski in alltasks) {
		var taskid = alltasks[taski];
		var task = this.taskForID(taskid);
		if (task != null) {
			spurious_abstractor.trace("TASK: "+task);
			task.setSyncStateToDeleted();
			task.commit();
			this.removeTaskWithID(task.id());
		}
	}
	
	this.categoryCollection().removeCategoryWithID(this.id());
	this.setSyncStateToDeleted();
	this.commit();
	
}


function CategorySorterFunction(a,b) {
	spurious_abstractor.trace("SORT FUNCTION");
	
	var res = a.creationTimeInSeconds() - b.creationTimeInSeconds();
	
	spurious_abstractor.trace(a.title());
	spurious_abstractor.trace(b.title());
	spurious_abstractor.trace(res);
	
	return res;
	
}


GTDCategory.prototype.stillExists = function() {
	if (this.categoryCollection().categoryWithIDExists(this.id())) {
		return true;
	} else {
		return false;	
	}
}

GTDCategory.prototype.sort = function(){
    
    
   this._tasks = this._tasks.sort(CategorySorterFunction);
    
}

GTDCategory.prototype.id = function(){
    return this._id;
}

GTDCategory.prototype.setPrettyName = function(name){
    this._prettyName = name;
}




GTDCategory.prototype.isSharedFromUser = function(){
    return this._sharedFromUser != null;
}


GTDCategory.prototype.sharedFromUser = function(){
    return this._sharedFromUser;
}


GTDCategory.prototype.prettyName = function(){
	
		if (this.id() == GTDCategoryCollection.UNCATEGORIZEDID) {
			return "Unfiled";
		}
		
		if (this._prettyName == null) {
			return "No name";
		}
	
    return this._prettyName;
}


GTDCategory.prototype.numberOfIncompleteTasks = function() {
	
	var alltasks = this.allTasks();
	var incompletecounter = 0;
	
	for (var i = 0; i < alltasks.length; i++) {
		task = alltasks[i];
		if (task.doneState() == false && task.isNote() == false) {
			incompletecounter++;
		}
	
	}
	
	return incompletecounter;
	
}


GTDCategory.prototype.numberOfNotes = function() {

	var alltasks = this.allTasks();
	var counter = 0;
	
	for (var i = 0; i < alltasks.length; i++) {
		task = alltasks[i];
		if (task.isNote() == true) {
			counter++;
		}
	
	}
	
	return counter;
	
}

GTDCategory.prototype.numberOfTasks = function() {
	return this._tasks.length - this.numberOfNotes();	
}

GTDCategory.prototype.allTaskIDs = function() {

	var alltasks = this.allTasks();
	var ids_out = [];
	
	for (var i = 0; i < alltasks.length; i++) {
		task = alltasks[i];
		ids_out.push(task.id());
	
	}
	
	return ids_out;

}

GTDCategory.prototype.addTask = function(task){
  	task.setCategoryID(this.id());
	this._tasks.push(task);
	
}

GTDCategory.prototype.allTasks = function(){
    
	return this._tasks;
	
}

GTDCategory.prototype.isUnfiledCategory = function() {
	if (this.id() == GTDCategoryCollection.UNCATEGORIZEDID) {
		return true;
	}
	
	return false;
}


GTDCategory.prototype.removeTaskWithID = function(id_in){

	var alltasks = this.allTasks();
	
	for (var i = 0; i < alltasks.length; i++) {
	
		if (alltasks[i].id() == id_in) {
			spurious_abstractor.trace("Removing task ("+alltasks[i].title()+") from category "+this.prettyName());
			alltasks.remove(i,i);
		}
	
	}
	
	//spurious_lawnchair.deleteTask(id_in);

}

GTDCategory.prototype.taskForID = function(id){
	
	alltasks = this.allTasks();
	
	
	for (var i = 0; i < alltasks.length; i++) {
	
		if (alltasks[i].id() == id) {
			return alltasks[i];
		}
	
	}
	
	return null;
	
}

//End Class Definition


// Start Class Definition...
function GTDCategoryCollection(){
  
  this._categories = [];
  
  
  
}



GTDCategoryCollection.prototype.categoryWithIDExists = function(c) {
	
	var cat = this.categoryForID(c);
	
	if (cat != null) {
		return true;
	}
	
	return false;
	
	
}

GTDCategoryCollection.prototype.ensureUnfiledCategory = function(){
	
	if (this.categoryForID(GTDCategoryCollection.UNCATEGORIZEDID) == null) {
		cat = new GTDCategory(GTDCategoryCollection.UNCATEGORIZEDID);
		this.addCategory(cat);
		cat.commit();
	}
	
}


GTDCategoryCollection.prototype.removeCategoryWithID = function(id_in){

	var allcats = this.allCategories();
	
	for (var i = 0; i < allcats.length; i++) {
	
		if (allcats[i].id() == id_in) {
			//air.trace("Removing task ("+alltasks[i].title()+") from category "+this.prettyName());
			allcats.remove(i,i);
		}
	
	}
	
	spurious_lawnchair.deleteCategory(id_in,true);

}

GTDCategoryCollection.prototype.doneLoading = function(){
	this.ensureUnfiledCategory();
}

GTDCategoryCollection.prototype.addCategory = function(cat){

	this._categories.push(cat);
	cat.setCategoryCollection(this);

}





GTDCategoryCollection.prototype.uncategorizedID = function(){
	return GTDCategoryCollection.UNCATEGORIZEDID;
}

GTDCategoryCollection.prototype.categoryForID = function(id){
	
	var allcats = this.allCategories();
	
	
	for (var i = 0; i < allcats.length; i++) {
	
		if (allcats[i].id() == id) {
			return allcats[i];
		}
	
	}
	
	spurious_abstractor.trace("NO CATEGORY FOR "+id);
	
	return null;
	
}


GTDCategoryCollection.prototype.sortFunction = function(a,b){
	
	
	if (a.isUnfiledCategory()) {
		return 1;
	}
	
	if (b.isUnfiledCategory()) {
		return -1;
	}
	
	var nameA = a.prettyName().toLowerCase();
	var nameB = b.prettyName().toLowerCase();
	
	if (nameA < nameB) {
  		return -1 
  	}
 
 	if (nameA > nameB) {
  		return 1
  	}
 
 	return 0 //default return value (no sorting)
	
}


GTDCategoryCollection.prototype.removeAll = function(){
	
	this._categories = [];
	
}

GTDCategoryCollection.prototype.firstCategory = function(vartoset){
	
	return this.allCategories()[0];
	
}



GTDCategoryCollection.prototype.taskExists = function(taskid) {
	
	if (this.taskForTaskID(taskid) == null) {
		return false;
	}
	
	return true;
	
}

GTDCategoryCollection.prototype.taskForTaskID = function(taskid) {


	var cati = 0;
	var allcats = this.allCategories();
	
	while (cati < allcats.length) {
		var cat  = allcats[cati];
		
		if (cat.taskForID(taskid) != null) {
			return cat.taskForID(taskid)
		}
		cati++;
	}

	return null;

}

GTDCategoryCollection.prototype.anyNewTasksOrCategories = function() {


	var cati = 0;
	var allcats = this.allCategories();
	
	while (cati < allcats.length) {
		var cat  = allcats[cati];
		
		if (cat.isNew()) {
			spurious_abstractor.trace("new cat!");
			return true;
		}
		
		var alltasks = cat.allTasks();
		var taski = 0;
		while (taski < alltasks.length) {
			var task = alltasks[taski];
			if (task.isNew()) {
				spurious_abstractor.trace("new task!");
				return true;
			}
			taski++;	
		}
		
		cati++;
	}

	return false;

}



GTDCategoryCollection.prototype.allCategoriesAsLookup = function(){
	
	var cati = 0;
	var allcats = this.allCategories();
	var cats_out = [];
	
	while (cati < allcats.length) {
		var cat  = allcats[cati];
		cats_out.push(cat.asLookup());
		cati++;	
	}
	
	return cats_out;
	
}


GTDCategoryCollection.prototype.allCategories = function(){


	return this._categories.sort(this.sortFunction);

	
	
}


//End Class Definition